# password.rb - password handling for Ruby
#
# $Id: password.rb,v 1.9 2002/10/03 05:27:02 ianmacd Exp $
# 
# Version : 0.3.0
# Author  : Ian Macdonald <ian@caliban.org>
#
# Copyright (C) 2002 Ian Macdonald
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

=begin

= NAME
Ruby/Password - password handling for Ruby
= SYNOPSIS

  require 'password'

  begin
    password = Password.get("New password")
    p = Password.new(password)
    p.check
  rescue Password::WeakPassword => reason
    puts reason
    retry
  end

= DESCRIPTION
Ruby/Password is a collection of password handling routines for Ruby,
including an interface to CrackLib for the purposes of testing password
strength.
= CLASS METHODS
--- Password.new(password)
    This creates a new instance of the Password class for ((|password|)).
--- Password.get(message = "Password")
    This obtains a password from the keyboard, using buffered line input.
    If ((|message|)) is given, it is used as the password prompt. This method
    also returns a new instance of the Password class.
--- Password.getc(message = "Password", mask = '*')
    This obtains a password from the keyboard, using unbuffered character
    input. If ((|message|)) is given, it is used as the password prompt.
    ((|mask|)) will be echoed to the terminal as each keypress is
    entered.
--- Password.echo(on = true, masked = false)
    This turns echoing to the terminal either on or off. If turning echoing
    off and ((|masked|)) is true, the keyboard is put into unbuffered mode,
    allowing the retrieval of characters one at a time. ((|masked|)) has no
    effect when turning echoing on.
--- Password.random(length = 8)
    This generates a random password by drawing bytes from the system
    entropy pool ((*/dev/urandom*)) and Base64-encoding them.
= INSTANCE METHODS
--- Password#crypt(type = Password::DES, salt = '')
    This encrypts the password, using the salt provided. If ((|salt|)) is
    not given, a random one will be generated and used. If ((|type|)) is set
    to ((|Password::MD5|)) and the underlying C library supports it, an MD5
    algorithm will be used to encrypt the password instead of the DES
    algorithm. If the C library does not support this, a ((*CryptError*))
    exception will be raised.
--- Password#check(dict = nil)
    This interfaces to LibCrack to check the strength of the password. If
    ((|dict|)) is given, it is the path to the CrackLib dictionary, minus the
    file's extension, so if the dictionary is located at
    /usr/lib/cracklib_dict.pwd, you would pass a ((|dict|)) of
    /usr/lib/cracklib_dict . If it is not given, the dictionary found at
    build time will be used.

    On success, ((*true*)) is returned. On failure, a Password::WeakPassword
    exception is raised.
= CONSTANTS
: Password::MD5
  When passed to the Password#crypt method, an MD5 algorithm will be used
  instead of the DES algorithm to encrypt the password.
: Password::DES
  By default, this is passed to the Password#crypt method to generate a DES-
  encrypted password.
= AUTHOR
Written by Ian Macdonald <ian@caliban.org>
= COPYRIGHT
 Copyright (C) 2002 Ian Macdonald

 This is free software; see the source for copying conditions.
 There is NO warranty; not even for MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.
= SEE ALSO
* ((<"Ruby/Password home page - http://www.caliban.org/ruby/"|URL:http://www.caliban.org/ruby/>))
* ((<crypt(3)>))
= BUGS
Send all bug reports, enhancement requests and patches to the
author.
= HISTORY
$Id: password.rb,v 1.9 2002/10/03 05:27:02 ianmacd Exp $

=end


require 'crack'
require 'termios'


class Password

  DES = true
  MD5 = false

  def Password.echo(on = true, masked = false)
    term = Termios::getattr($stdin)

    if on
      term.c_lflag |= (Termios::ECHO | Termios::ICANON)
    else # off
      term.c_lflag &= ~Termios::ECHO
      term.c_lflag &= ~Termios::ICANON if masked
    end

    Termios::setattr($stdin, Termios::TCSANOW, term)
  end

  def Password.get(message = "Password")
    begin
      Password.echo false
      print message, ": "
      Password.new($stdin.gets.chomp)

    ensure
      Password.echo true
      print "\n"
    end
  end

  def Password.getc(message = "Password", mask = '*')
    begin
      Password.echo(false, true)
      print message, ": "
      password = ""

      while (char = $stdin.getc) != 10 # \n
	putc mask
	password << char
      end

    ensure
      Password.echo true
      print "\n"
    end

    return password
  end

  def Password.random(length = 8)
    rand_data = nil
    File.open("/dev/urandom") { |f| rand_data = f.read(length) }

    # Base64 encode it
    return [rand_data].pack('m')[0..length-1]
  end

  def crypt(type = DES, salt = '')
    salt = Password.random(type ? 2 : 8) if salt.empty?

    # (Linux glibc2 interprets a salt prefix of '$1$' as a call to use MD5
    # instead of DES when calling crypt(3))
    salt = '$1$' + salt if type == MD5

    # pass to crypt in class String (our grandparent class)
    output = super(salt)

    # raise an exception if MD5 was wanted, but result is not recognisable
    raise CryptError, 'MD5 not implemented' if type == MD5 && output !~ /^\$1\$/

    return output
  end

end

class CryptError < RuntimeError
  def initialize(message)
    super
  end
end
